/*
 * nodeMDSimJoin.c
 *
 *  Created on: Mar 8, 2011
 *      Author: postgres
 */

#include "postgres.h"

#include "nodes/execnodes.h"
#include "nodes/plannodes.h"
#include "executor/nodeMDSimJoin.h"
#include "utils/tuplestore.h"
#include <math.h>;

TupleTableSlot *
ExecMDSimJoin(MDSimJoinState *node)
{
	PlanState  *innerPlan;
	PlanState  *outerPlan;
	TupleTableSlot *outerTupleSlot;
	TupleTableSlot *innerTupleSlot;
	List	   *joinqual;
	List	   *otherqual;
	ExprContext *econtext;

	/*
	 * get information from the node
	 */
	EMDSJ1_printf("getting info from node");

	joinqual = node->js.joinqual;
	otherqual = node->js.ps.qual;
	outerPlan = outerPlanState(node);
	innerPlan = innerPlanState(node);
	econtext = node->js.ps.ps_ExprContext;
}

MDSimJoinState *
ExecInitMDSimJoin(MDSimJoin *node, EState *estate, int eflags)
{
	MDSimJoinState *mdsjstate;

	/* check for unsupported flags */
	Assert(!(eflags & (EXEC_FLAG_BACKWARD | EXEC_FLAG_MARK)));

	MDSJ1_printf("ExecInitMDSimJoin: %s\n",
			   "initializing node");

	/*
	 * create state structure
	 */
	mdsjstate = makeNode(MDSimJoinState);
	mdsjstate->js.ps.plan = (Plan *) node;
	mdsjstate->js.ps.state = estate;

	/*
	 * Miscellaneous initialization
	 *
	 * create expression context for nodes
	 */
	ExecAssignExprContext(estate, &msjstate->js.ps);

	/*
	 * initialize child expressions
	 */
	mdsjstate->js.ps.targetlist = (List *)
		ExecInitExpr((Expr *) node->join.plan.targetlist,
					 (PlanState *) mdsjstate);
	mdsjstate->js.ps.qual = (List *)
		ExecInitExpr((Expr *) node->join.plan.qual,
					 (PlanState *) mdsjstate);
	mdsjstate->js.jointype = node->join.jointype;
	mdsjstate->js.joinqual = (List *)
		ExecInitExpr((Expr *) node->join.joinqual,
					 (PlanState *) mdsjstate);

	/*
	 * initialize child nodes
	 */
	outerPlanState(mdsjstate) = ExecInitNode(outerPlan(node), estate, eflags);
	innerPlanState(mdsjstate) = ExecInitNode(innerPlan(node), estate,
										   eflags | EXEC_FLAG_REWIND);

#define MDSIMJOIN_NSLOTS 2

	/*
	 * tuple table initialization
	 */
	ExecInitResultTupleSlot(estate, &mdsjstate->js.ps);

	switch (node->join.jointype)
	{
		case JOIN_INNER:
		case JOIN_IN:
			break;
		case JOIN_LEFT:
			mdsjstate->mdsj_NullInnerTupleSlot =
				ExecInitNullTupleSlot(estate,
								 ExecGetResultType(innerPlanState(nlstate)));
			break;
		default:
			elog(ERROR, "unrecognized join type: %d",
				 (int) node->join.jointype);
	}

	/*
	 * initialize tuple type and projection info
	 */
	ExecAssignResultTypeFromTL(&mdsjstate->js.ps);
	ExecAssignProjectionInfo(&mdsjstate->js.ps, NULL);

	/*
	 * finally, wipe the current outer tuple clean.
	 */
	mdsjstate->js.ps.ps_OuterTupleSlot = NULL;
	mdsjstate->js.ps.ps_TupFromTlist = false;
	mdsjstate->mdsj_NeedNewOuter = true;
	mdsjstate->mdsj_MatchedOuter = false;

	MDSJ1_printf("ExecInitMDSimJoin: %s\n",
			   "node initialized");

	/*
	 * get number of partitions and windows
	 */
	mdsjstate->numPartitions = floor(sqrt((node->availMemory -
			sizeof(Tuplestorestate))/sizeof(Tuplestorestate)));
	mdsjstate->numwindows = numPartitions * (numPartitions - 1);
	mdsjstate->partitions = (Tuplestorestate *)
			palloc(sizeof(Tuplestorestate) * numPartitions);
	mdsjstate->windows = (Tuplestorestate *)
			palloc(sizeof(Tuploestorestate) * numWindows);
	mdsjstate->pivots = (Datum *) palloc(sizeof(Datum) * numPartitions);

	mdsjstate->epsilon = node->epsilon;

	return mdsjstate;
}

int ExecCountSlotsMDSimJoin(MDSimJoin *node)
{
	return ExecCountSlotsNode(outerPlan(node)) +
		ExecCountSlotsNode(innerPlan(node)) +
		MDSIMJOIN_NSLOTS;
}

extern void ExecEndMDSimJoin(MDSimJoinState *node)
{

}
